import org.lwjgl.Sys;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GL11;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.util.Random;

public class Renderer {
    //Timers
    long fps, lastFPS, lastFrame, ticks;

    //Debug JFrame
    DbgFrame dbgframe;

    public void updateFPS() {
        if (Sys.getTime() - lastFPS > 1000) {
            Display.setTitle("FosEngine FPS: " + fps);
            fps = 0; //reset the FPS counter
            lastFPS += 1000; //add one second
        }
        fps++;

    }


    public int getDeltaTime() {
        long time = Sys.getTime();
        int delta = (int) (time - lastFrame);
        lastFrame = time;

        return delta;
    }


    Renderer() {
        try {
            //lwjgl init
            Display.setDisplayMode(new DisplayMode(800, 600));
            Display.create();


            //OpenGL init
            GL11.glMatrixMode(GL11.GL_PROJECTION);
            GL11.glLoadIdentity();
            GL11.glFrustum(-400, 400, 300, -300, 600, 15000);
            GL11.glEnable(GL11.GL_DEPTH_TEST);
            GL11.glMatrixMode(GL11.GL_MODELVIEW);
            GL11.glEnable(GL11.GL_TEXTURE_2D);
            GL11.glEnable(GL11.GL_BLEND);
            GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
            GL11.glDepthFunc(GL11.GL_LEQUAL);
            GL11.glHint(GL11.GL_PERSPECTIVE_CORRECTION_HINT, GL11.GL_NICEST);
            GL11.glShadeModel(GL11.GL_SMOOTH);
            GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL11.GL_CLAMP);
            GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL11.GL_CLAMP);

            //init lights
            float lightAmbient[] = {0.5f, 0.5f, 0.5f, 1.0f};
            float lightDiffuse[] = {1f, 0.8f, 0.8f, 1.0f};
            float lightPosition[] = {1f, -1f, 0.5f, 0.0f};
            ByteBuffer temp = ByteBuffer.allocateDirect(16);
            temp.order(ByteOrder.nativeOrder());
            GL11.glLight(GL11.GL_LIGHT1, GL11.GL_AMBIENT, (FloatBuffer) temp.asFloatBuffer().put(lightAmbient).flip());
            GL11.glLight(GL11.GL_LIGHT1, GL11.GL_DIFFUSE, (FloatBuffer) temp.asFloatBuffer().put(lightDiffuse).flip());
            GL11.glLight(GL11.GL_LIGHT1, GL11.GL_POSITION, (FloatBuffer) temp.asFloatBuffer().put(lightPosition).flip());
            GL11.glEnable(GL11.GL_LIGHT1);
            GL11.glEnable(GL11.GL_LIGHTING);


            //initial positioning
            int zpos = -2000, ypos = 800, xpos = -3000;
            float yrotrad, xrotrad, rotx = 0, rotz = 0, roty = 0;
            int dx, dy;
            
            
            
            //init fps counter
            lastFPS = Sys.getTime();

            //init debug frame
            dbgframe = new DbgFrame();
            new Thread(dbgframe).start();
            
            
            //init world
            World world = new World();
           world.populateGround();
            world.addBlock(3,4,5, World.dirt);
            Random random = new Random();
            for(int i = 1000; i>=0; i--){
                world.addBlock(random.nextInt(40), random.nextInt(20), random.nextInt(40), random.nextInt(2)+1);
                
            }
            world.refreshBuffer();
            world.generateWorldRenderList();
            
            
            //Main render loop
            while (dbgframe.isRunning()) {
                updateFPS();
                ticks = getDeltaTime();
                dbgframe.setTicks("Ticks="+ticks);

                //calculate radian values
                yrotrad = roty / 180f * 3.141592654f;
                xrotrad = rotx / 180f * 3.141592654f;

                //handle keyboard events
                if (Keyboard.isKeyDown(Keyboard.KEY_D)) {

                    xpos -= Math.cos(yrotrad)*ticks*2;
                    zpos -= Math.sin(yrotrad)*ticks*2;
                    dbgframe.setDgbX(Integer.toString(xpos));
                }
                if (Keyboard.isKeyDown(Keyboard.KEY_A)) {
                    xpos += Math.cos(yrotrad)*ticks*2;
                    zpos += Math.sin(yrotrad)*ticks*2;
                    dbgframe.setDgbX(Integer.toString(xpos));
                    dbgframe.setDgbZ(Integer.toString(zpos));
                }
                if (Keyboard.isKeyDown(Keyboard.KEY_S)) {
                    xpos += Math.sin(yrotrad)*ticks*2;
                    zpos -= Math.cos(yrotrad)*ticks*2;
                    ypos -= Math.sin(xrotrad)*ticks*2;
                    dbgframe.setDgbZ(Integer.toString(zpos));
                    dbgframe.setDgbX(Integer.toString(xpos));
                    dbgframe.setDgbY(Integer.toString(ypos));
                }
                if (Keyboard.isKeyDown(Keyboard.KEY_W)) {
                    xpos -= Math.sin(yrotrad)*ticks*2;
                    zpos += Math.cos(yrotrad)*ticks*2;
                    ypos += Math.sin(xrotrad)*ticks*2;
                    dbgframe.setDgbZ(Integer.toString(zpos));
                    dbgframe.setDgbX(Integer.toString(xpos));
                    dbgframe.setDgbY(Integer.toString(ypos));
                }

                if (Keyboard.isKeyDown(Keyboard.KEY_LSHIFT)) {
                    ypos -= ticks*2;
                    dbgframe.setDgbY(Integer.toString(ypos));
                }
                if (Keyboard.isKeyDown(Keyboard.KEY_SPACE)){
                    ypos += ticks*2;
                    dbgframe.setDgbY(Integer.toString(ypos));
                }

                if(Keyboard.isKeyDown(Keyboard.KEY_LCONTROL) && Keyboard.isKeyDown(Keyboard.KEY_TAB))
                    Mouse.setGrabbed(false);
                if(Mouse.isButtonDown(0) && !Mouse.isGrabbed())
                    Mouse.setGrabbed(true);
                if(Keyboard.isKeyDown(Keyboard.KEY_ESCAPE) && Mouse.isGrabbed())
                    System.exit(0);
                //Mouse
                if(Mouse.isGrabbed()){
                     dx = Mouse.getDY();
                     dy = Mouse.getDX();
                    if((rotx > -90 || dx > 0) && (rotx < 90 || dx < 0))
                        rotx += dx/2f; //set the xrot to xrot with the additionof the difference in the y position
                    roty += dy/2f;    //set the xrot to yrot with the additionof the difference in the x position
                    
                }

                //reset rotaton
                if (roty > 360) roty -= 360;
                if (rotz > 360) rotx -= 360;


                GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);

                GL11.glLoadIdentity();

                
                
                //position brush
                GL11.glRotatef(rotx, 1f, 0f, 0f);
                GL11.glRotatef(roty, 0f, 1f, 0f);
                GL11.glTranslatef(xpos, ypos, zpos);

                //lights
                GL11.glLight(GL11.GL_LIGHT1, GL11.GL_POSITION, (FloatBuffer) temp.asFloatBuffer().put(lightPosition).flip());


                //render world
                world.render();




                Display.sync(60);
                Display.update();
            }

            Display.destroy();
            System.exit(0);
        } catch (Exception e) {
            e.printStackTrace();
        }


    }

}
